@@ -64,31 +64,18 @@ enum
 	SELECTION_CHANGED,
 	ITEM_ACTIVATED,
 	LAST_SIGNAL
-};
-					  
-enum
-{
-	TITLE_COLUMN,
-	WIDGET_COLUMN,
-	ROW_VISIBLE,
-	N_COLUMNS
-};
+};		
 
 struct _GladeInspectorPrivate
 {
 	GtkWidget    *view;
-	GtkTreeStore *model;
 	GtkTreeModel *filter;
-	GtkTreeIter   actions_iter;
-	GtkTreeIter   widgets_iter;
-	GtkTreeIter   objects_iter;
 
 	GladeProject *project;
 
 	GtkWidget    *entry;
-        GCompletion  *completion;
-	guint         idle_complete;
-	guint         idle_filter;
+	GCompletion * completion;
+	guint idle_complete;
 	gboolean      search_disabled;
 };
 
@@ -108,11 +95,6 @@ static void     selection_changed_cb        (GtkTreeSelection  *selection,
 static gint     button_press_cb 	    (GtkWidget         *widget,
 					     GdkEventButton    *event,
 					     GladeInspector    *inspector);
-static void     update_model                (GladeInspector    *inspector);
-static void     disconnect_project_signals  (GladeInspector    *inspector,
-			 	             GladeProject      *project);
-static void     connect_project_signals     (GladeInspector    *inspector,
-			 	             GladeProject      *project);
 
 G_DEFINE_TYPE (GladeInspector, glade_inspector, GTK_TYPE_VBOX)
 
@@ -210,95 +192,106 @@ glade_inspector_class_init (GladeInspectorClass *klass)
 	g_type_class_add_private (klass, sizeof (GladeInspectorPrivate));
 }
 
-static void
-refilter_inspector (GladeInspector *inspector)
+static gboolean
+find_in_string_insensitive (const gchar *_haystack,
+			    const gchar *_needle)
 {
-	GladeInspectorPrivate *priv = inspector->priv;
+	gboolean visible;
+	gchar *haystack = g_utf8_casefold (_haystack, -1);
+	gchar *needle   = g_utf8_casefold (_needle, -1);
 
-	g_object_ref (priv->filter);
+	visible = strstr (haystack, needle) != NULL;
 
-	gtk_tree_view_collapse_all (GTK_TREE_VIEW (priv->view));
-	gtk_tree_view_set_model (GTK_TREE_VIEW (priv->view), NULL);
-	gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (priv->filter));
-	gtk_tree_view_set_model (GTK_TREE_VIEW (priv->view), priv->filter);
-	gtk_tree_view_expand_all (GTK_TREE_VIEW (priv->view));
+	g_free (haystack);
+	g_free (needle);
 
-	g_object_unref (priv->filter);
+	return visible;
 }
 
 static gboolean
-search_filter_idle (GladeInspector *inspector)
+glade_inspector_visible_func (GtkTreeModel* model,
+                              GtkTreeIter* parent,
+                              gpointer data)
 {
+	GladeInspector* inspector = data;
 	GladeInspectorPrivate *priv = inspector->priv;
-	GladeWidget           *selected;
-        const gchar           *str;
 
-        str = gtk_entry_get_text (GTK_ENTRY (priv->entry));
+	GtkTreeIter iter;
+	
+	gboolean retval = FALSE;
 
-	/* filter the model from here ! */
-	refilter_inspector (inspector);
-
-	if ((selected = glade_project_get_widget_by_name (priv->project, NULL, str)) != NULL)
+	if (priv->search_disabled)
+		return TRUE;
+	
+	if (gtk_tree_model_iter_children (model, &iter, parent))
 	{
-		GtkTreeSelection *selection;
-		GtkTreeIter      *iter;
-
-		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->view));
-		gtk_tree_selection_unselect_all (selection);
-
-		if ((iter = glade_util_find_iter_by_widget (GTK_TREE_MODEL (inspector->priv->filter),
-							    selected,
-							    WIDGET_COLUMN)) != NULL)
+		do
 		{
-			gtk_tree_selection_select_iter (selection, iter);
-			gtk_tree_iter_free (iter);
+			retval = glade_inspector_visible_func (model, &iter, data);
 		}
+		while (gtk_tree_model_iter_next (model, &iter) && !retval);
+	}
+	if (!retval)
+	{
+		const gchar* text = gtk_entry_get_text (GTK_ENTRY(priv->entry));
+		gchar* widget_name;
+		
+		gtk_tree_model_get (model, parent, GLADE_PROJECT_MODEL_COLUMN_NAME,
+		                    &widget_name, -1);
+
+		retval = find_in_string_insensitive (widget_name, text);
+
+		g_free (widget_name);
 	}
 
-        priv->idle_filter = 0;
+	return retval;
+}
 
-        return FALSE;
+static void
+glade_inspector_filter (GladeInspector* inspector)
+{
+	GladeInspectorPrivate *priv = inspector->priv;
+
+	if (!priv->search_disabled)
+	{
+		gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER(priv->filter));
+		gtk_tree_view_expand_all (GTK_TREE_VIEW (priv->view));
+	}
+		
 }
 
 static void
 search_entry_changed_cb (GtkEntry *entry,
 			 GladeInspector *inspector)
 {
-	GladeInspectorPrivate *priv = inspector->priv;
-
-        if (!priv->search_disabled && !priv->idle_filter) {
-                priv->idle_filter =
-                        g_idle_add ((GSourceFunc) search_filter_idle, inspector);
-        }
+	glade_inspector_filter (inspector);
 }
 
 static gboolean
 search_complete_idle (GladeInspector *inspector)
 {
 	GladeInspectorPrivate *priv = inspector->priv;
-        const gchar           *str;
-        gchar                 *completed = NULL;
-        GList                 *list;
-        gsize                  length;
+	const gchar           *str;
+	gchar                 *completed = NULL;
+	GList                 *list;
+	gsize                  length;
 
-        str = gtk_entry_get_text (GTK_ENTRY (priv->entry));
+	str = gtk_entry_get_text (GTK_ENTRY (priv->entry));
 
-        list = g_completion_complete (priv->completion, str, &completed);
-        if (completed) {
-                length = strlen (str);
+	list = g_completion_complete (priv->completion, str, &completed);
+	if (completed) {
+		length = strlen (str);
 
-                gtk_entry_set_text (GTK_ENTRY (priv->entry), completed);
-                gtk_editable_set_position (GTK_EDITABLE (priv->entry), length);
-                gtk_editable_select_region (GTK_EDITABLE (priv->entry),
-                                            length, -1);
+		gtk_entry_set_text (GTK_ENTRY (priv->entry), completed);
+		gtk_editable_set_position (GTK_EDITABLE (priv->entry), length);
+		gtk_editable_select_region (GTK_EDITABLE (priv->entry),
+		                            length, -1);
 		g_free (completed);
-        }
+	}
 
-	refilter_inspector (inspector);
-	
-        priv->idle_complete = 0;
+	priv->idle_complete = 0;
 
-        return FALSE;
+	return FALSE;
 }
 
 static void
@@ -310,11 +303,11 @@ search_entry_text_inserted_cb (GtkEntry       *entry,
 {
 	GladeInspectorPrivate *priv = inspector->priv;
 
-        if (!priv->search_disabled && !priv->idle_complete) {
-                priv->idle_complete =
-                        g_idle_add ((GSourceFunc) search_complete_idle,
-                                    inspector);
-        }
+	if (!priv->search_disabled && !priv->idle_complete) {
+		priv->idle_complete =
+			g_idle_add ((GSourceFunc) search_complete_idle,
+			            inspector);
+	}
 }
 
 static gboolean
@@ -323,23 +316,23 @@ search_entry_key_press_event_cb (GtkEntry    *entry,
                                  GladeInspector *inspector)
 {
 	GladeInspectorPrivate *priv = inspector->priv;
-        const gchar           *str;
+	const gchar           *str;
 
-        if (event->keyval == GDK_Tab) 
+	if (event->keyval == GDK_Tab)
 	{
-                if (event->state & GDK_CONTROL_MASK) 
+		if (event->state & GDK_CONTROL_MASK)
 		{
-                        gtk_widget_grab_focus (priv->view);
-                }
+			gtk_widget_grab_focus (priv->view);
+		}
 		else
 		{
-                        gtk_editable_set_position (GTK_EDITABLE (entry), -1);
-                        gtk_editable_select_region (GTK_EDITABLE (entry), -1, -1);
-                }
-                return TRUE;
-        }
+			gtk_editable_set_position (GTK_EDITABLE (entry), -1);
+			gtk_editable_select_region (GTK_EDITABLE (entry), -1, -1);
+		}
+		return TRUE;
+	}
 
-        if (event->keyval == GDK_Return || event->keyval == GDK_KP_Enter) 
+	if (event->keyval == GDK_Return || event->keyval == GDK_KP_Enter)
 	{
 		GladeWidget *widget;
 		GList       *list;
@@ -350,97 +343,15 @@ search_entry_key_press_event_cb (GtkEntry    *entry,
 		{
 			widget = glade_widget_get_from_gobject (list->data);
 
-                        gtk_entry_set_text (GTK_ENTRY (entry), widget->name);
+			gtk_entry_set_text (GTK_ENTRY (entry), widget->name);
 
-                        gtk_editable_set_position (GTK_EDITABLE (entry), -1);
-                        gtk_editable_select_region (GTK_EDITABLE (entry), -1, -1);
+			gtk_editable_set_position (GTK_EDITABLE (entry), -1);
+			gtk_editable_select_region (GTK_EDITABLE (entry), -1, -1);
 		}
 		return TRUE;
         }
-	
-        return FALSE;
-}
 
-static gboolean
-find_in_string_insensitive (const gchar *_haystack,
-			    const gchar *_needle)
-{
-	gboolean visible;
-	gchar *haystack = g_ascii_strdown (_haystack, -1);
-	gchar *needle   = g_ascii_strdown (_needle, -1);
-
-	visible = strstr (haystack, needle) != NULL;
-
-	g_free (haystack);
-	g_free (needle);
-	
-	return visible;
-}
-
-static gboolean
-search_children_visible (GladeWidget *widget, 
-			 const gchar *needle)
-{
-	GList     *l, *children;
-	gboolean   visible = FALSE;
-
-	children = glade_widget_adaptor_get_children (widget->adaptor, widget->object);
-
-	for (l = children; l; l = l->next)
-	{
-		GObject     *child = l->data;
-		GladeWidget *gchild = glade_widget_get_from_gobject (child);
-
-		if (!gchild)
-			continue;
-
-		if ((visible = search_children_visible (gchild, needle)))
-			break;		
-	}
-	g_list_free (children);
-
-	if (!visible)
-		visible = find_in_string_insensitive (widget->name, needle);
-
-	return visible;
-}
-
-static gboolean
-filter_visible_func (GtkTreeModel *model,
-		     GtkTreeIter *iter,
-		     GladeInspector *inspector)
-{
-	GladeInspectorPrivate *priv = inspector->priv;
-	GladeWidget           *widget = NULL;
-        const gchar           *str;
-	gboolean               visible;
-
-	if (!priv->project)
-		return FALSE;
-
-	gtk_tree_model_get (model, iter, 
-			    WIDGET_COLUMN, &widget, 
-			    -1);
-	if (!widget || priv->search_disabled)
-		return TRUE;
-
-        if ((str = gtk_entry_get_text (GTK_ENTRY (priv->entry))) == NULL)
-		return TRUE;
-	
-	/* return true for any child widget with the same text (child nodes are
-	 * not visible without thier parents
-	 */
-	visible = search_children_visible (widget, str);
-
-	return visible;
-}
-
-static const gchar *
-search_complete_func (GObject *object)
-{
-	GladeWidget *widget = glade_widget_get_from_gobject (object);
-	g_assert (widget);
-	return glade_widget_get_name (widget);
+	return FALSE;
 }
 
 static void
@@ -453,6 +364,14 @@ widget_font_desc_set_style (GtkWidget *widget, PangoStyle style)
 	pango_font_description_free (font_desc);
 }
 
+static const gchar *
+search_complete_func (GObject *object)
+{
+	GladeWidget *widget = glade_widget_get_from_gobject (object);
+	g_assert (widget);
+	return glade_widget_get_name (widget);
+}
+
 static void
 search_entry_update (GladeInspector *inspector)
 {
@@ -511,46 +430,35 @@ glade_inspector_init (GladeInspector *inspector)
 	priv->project = NULL;
 
 	priv->entry = gtk_entry_new ();
+	
 	search_entry_update (inspector);
 	gtk_widget_show (priv->entry);
 	gtk_box_pack_start (GTK_BOX (inspector), priv->entry, FALSE, FALSE, 2);
 
-        g_signal_connect (priv->entry, "key-press-event",
-                          G_CALLBACK (search_entry_key_press_event_cb),
-                          inspector);
+	g_signal_connect (priv->entry, "changed",
+	                  G_CALLBACK (search_entry_changed_cb),
+	                  inspector);
 
-        g_signal_connect (priv->entry, "changed",
-                          G_CALLBACK (search_entry_changed_cb),
-                          inspector);
+	g_signal_connect (priv->entry, "key-press-event",
+	                  G_CALLBACK (search_entry_key_press_event_cb),
+	                  inspector);
 
-        g_signal_connect (priv->entry, "insert-text",
-                          G_CALLBACK (search_entry_text_inserted_cb),
-                          inspector);
+	g_signal_connect_after (priv->entry, "insert-text",
+	                        G_CALLBACK (search_entry_text_inserted_cb),
+	                        inspector);
+
+	g_signal_connect (priv->entry, "focus-in-event",
+	                  G_CALLBACK (search_entry_focus_in_cb),
+	                  inspector);
 
-        g_signal_connect (priv->entry, "focus-in-event",
-                          G_CALLBACK (search_entry_focus_in_cb),
-                          inspector);
-        
 	g_signal_connect (priv->entry, "focus-out-event",
-                          G_CALLBACK (search_entry_focus_out_cb),
-                          inspector);
-	
-        priv->completion = g_completion_new ((GCompletionFunc) search_complete_func);
+	                  G_CALLBACK (search_entry_focus_out_cb),
+	                  inspector);
+
+	priv->completion = g_completion_new ((GCompletionFunc) search_complete_func);
 
 	priv->view = gtk_tree_view_new ();
-
-	priv->model  = gtk_tree_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_BOOLEAN);
-	priv->filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (priv->model), NULL);
-
-	gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (priv->filter),
-						(GtkTreeModelFilterVisibleFunc)filter_visible_func,
-						inspector, NULL);
-
-	gtk_tree_view_set_model (GTK_TREE_VIEW (priv->view), priv->filter);
-        gtk_tree_view_set_enable_search (GTK_TREE_VIEW (priv->view), FALSE);
-
-	g_object_unref (G_OBJECT (priv->model));
-	g_object_unref (G_OBJECT (priv->filter));
+	gtk_tree_view_set_enable_search (GTK_TREE_VIEW (priv->view), FALSE);
 
  
 	add_columns (GTK_TREE_VIEW (priv->view));
@@ -587,11 +495,11 @@ glade_inspector_init (GladeInspector *inspector)
 static void
 glade_inspector_dispose (GObject *object)
 {
-	GladeInspectorPrivate *priv = GLADE_INSPECTOR_GET_PRIVATE (object);
+	GladeInspector *inspector = GLADE_INSPECTOR(object);
+	GladeInspectorPrivate *priv = inspector->priv;
 	
 	if (priv->project)
 	{
-		disconnect_project_signals (GLADE_INSPECTOR (object), priv->project);
 		g_object_unref (priv->project);
 		priv->project = NULL;
 	}
@@ -605,202 +513,6 @@ glade_inspector_finalize (GObject *object)
 	G_OBJECT_CLASS (glade_inspector_parent_class)->finalize (object);
 }
 
-static void
-fill_model (GtkTreeStore *model,
-	    const GList  *widgets,
-	    GtkTreeIter  *parent_iter)
-{
-	GList *children, *l;
-	GtkTreeIter       iter;
-
-	for (l = (GList *) widgets; l; l = l->next)
-	{
-		GladeWidget *widget;
-		
-		widget = glade_widget_get_from_gobject ((GObject *) l->data);
-
-		if (widget != NULL)
-		{
-			gtk_tree_store_append (model, &iter, parent_iter);
-			gtk_tree_store_set    (model, &iter, WIDGET_COLUMN, widget, -1);
-
-			children = glade_widget_adaptor_get_children (widget->adaptor, widget->object);	
-		
-			if (children != NULL)
-			{
-				GtkTreeIter *copy = NULL;
-
-				copy = gtk_tree_iter_copy (&iter);
-				
-				fill_model (model, children, copy);
-				
-				gtk_tree_iter_free (copy);
-
-				g_list_free (children);
-			}
-		}
-	}
-}
-
-static void
-update_model (GladeInspector *inspector)
-{
-	GladeInspectorPrivate *priv = inspector->priv;
-	GList *l, *toplevels = NULL;
-	
-	gtk_tree_store_clear (priv->model);
-
-	g_completion_clear_items (priv->completion);
-
-	if (!priv->project)
-		return;
-
-	g_completion_add_items (priv->completion, (GList *)glade_project_get_objects (priv->project));
-
-
-	/* make a list of only the actions and groups */
-	for (l = (GList *) glade_project_get_objects (priv->project); l; l = l->next)
-	{
-		GObject     *object  = G_OBJECT (l->data);
-		GladeWidget *gwidget = glade_widget_get_from_gobject (object);
-		g_assert (gwidget);
-
-		if (gwidget->parent == NULL && 
-		    (GTK_IS_ACTION (object) || GTK_IS_ACTION_GROUP (object)))
-			toplevels = g_list_prepend (toplevels, object);
-	}
-	toplevels = g_list_reverse (toplevels);
-
-	/* recursively fill model */
-	gtk_tree_store_append (priv->model, &priv->actions_iter, NULL);
-	gtk_tree_store_set    (priv->model, &priv->actions_iter, TITLE_COLUMN, _("Actions"), -1);
-	fill_model (priv->model, toplevels, &priv->actions_iter);
-	toplevels = (g_list_free (toplevels), NULL);
-
-	/* make a list of only the toplevel window widgets */
-	for (l = (GList *) glade_project_get_objects (priv->project); l; l = l->next)
-	{
-		GObject     *object  = G_OBJECT (l->data);
-		GladeWidget *gwidget = glade_widget_get_from_gobject (object);
-		g_assert (gwidget);
-
-		if (gwidget->parent == NULL && GTK_IS_WIDGET (object))
-			toplevels = g_list_prepend (toplevels, object);
-	}
-	toplevels = g_list_reverse (toplevels);
-
-	/* recursively fill model */
-	gtk_tree_store_append (priv->model, &priv->widgets_iter, NULL);
-	gtk_tree_store_set    (priv->model, &priv->widgets_iter, TITLE_COLUMN, _("Widgets"), -1);
-	fill_model (priv->model, toplevels, &priv->widgets_iter);
-	toplevels = (g_list_free (toplevels), NULL);
-
-	/* make a list of only the toplevel non-window widgets */
-	for (l = (GList *) glade_project_get_objects (priv->project); l; l = l->next)
-	{
-		GObject     *object  = G_OBJECT (l->data);
-		GladeWidget *gwidget = glade_widget_get_from_gobject (object);
-		g_assert (gwidget);
-
-		if (gwidget->parent == NULL && !GTK_IS_WIDGET (object) && 
-		    !GTK_IS_ACTION (object) && !GTK_IS_ACTION_GROUP (object))
-			toplevels = g_list_prepend (toplevels, object);
-	}
-	toplevels = g_list_reverse (toplevels);
-
-	/* recursively fill model */
-	gtk_tree_store_append (priv->model, &priv->objects_iter, NULL);
-	gtk_tree_store_set    (priv->model, &priv->objects_iter, TITLE_COLUMN, _("Objects"), -1);
-	fill_model (priv->model, toplevels, &priv->objects_iter);
-	toplevels = (g_list_free (toplevels), NULL);
-}
-
-static void
-project_add_widget_cb (GladeProject   *project,
-		       GladeWidget    *widget,
-		       GladeInspector *inspector)
-{
-	GladeInspectorPrivate *priv = inspector->priv;
-	GladeWidget *parent_widget;
-	GtkTreeIter widget_iter, *parent_iter = NULL;
-	GList *l;
-
-	g_completion_clear_items (priv->completion);
-	g_completion_add_items (priv->completion, (GList *)glade_project_get_objects (priv->project));
-
-	parent_widget = glade_widget_get_parent (widget);
-	if (parent_widget != NULL)
-		parent_iter = glade_util_find_iter_by_widget (GTK_TREE_MODEL (inspector->priv->model),
-							      parent_widget,
-							      WIDGET_COLUMN);
-	/* we have to add parents first, then children */
-	if (!parent_iter && parent_widget)
-		return;
-	
-	if (!parent_iter)
-	{
-		if (GTK_IS_ACTION (widget->object) || GTK_IS_ACTION_GROUP (widget->object))
-			parent_iter = &inspector->priv->actions_iter;
-		else if (GTK_IS_WIDGET (widget->object))
-			parent_iter = &inspector->priv->widgets_iter;
-		else
-			parent_iter = &inspector->priv->objects_iter;
-	}
-
-	gtk_tree_store_append (inspector->priv->model, &widget_iter, parent_iter);	
-	gtk_tree_store_set    (inspector->priv->model, &widget_iter, WIDGET_COLUMN, widget, -1);
-	
-	fill_model (inspector->priv->model,
-		    l = glade_widget_adaptor_get_children (widget->adaptor, widget->object),
-		    &widget_iter);
-
-	g_list_free (l);
-
-	gtk_tree_view_expand_all (GTK_TREE_VIEW (inspector->priv->view));
-}
-
-static void
-project_remove_widget_cb (GladeProject   *project,
-			  GladeWidget    *widget,
-			  GladeInspector *inspector)
-{
-	GladeInspectorPrivate *priv = inspector->priv;
-	GtkTreeIter *iter;
-
-	g_completion_clear_items (priv->completion);
-	g_completion_add_items (priv->completion, (GList *)glade_project_get_objects (priv->project));
-
-	iter = glade_util_find_iter_by_widget (GTK_TREE_MODEL (inspector->priv->model),
-					       widget,
-					       WIDGET_COLUMN);
-	if (iter)
-	{
-		gtk_tree_store_remove (inspector->priv->model, iter);
-		gtk_tree_iter_free (iter);
-	}
-}
-
-static void
-project_widget_name_changed_cb (GladeProject   *project,
-			        GladeWidget    *widget,
-				GladeInspector *inspector)
-{
-	GtkTreeModel *model;
-	GtkTreeIter  *iter;
-	GtkTreePath  *path;
-
-	model = GTK_TREE_MODEL (inspector->priv->model);
-
-	iter = glade_util_find_iter_by_widget (model, widget, WIDGET_COLUMN);
-
-	if (iter)
-	{
-		path = gtk_tree_model_get_path (model, iter);
-		gtk_tree_model_row_changed (model, path, iter);
-		gtk_tree_iter_free (iter);
-	}
-}
-
 static void
 project_selection_changed_cb (GladeProject     *project,
 			      GladeInspector   *inspector)
@@ -811,7 +523,7 @@ project_selection_changed_cb (GladeProject     *project,
 	GtkTreeIter      *iter;
 	GtkTreePath      *path, *ancestor_path;
 	GList            *list;
-	
+
 	g_return_if_fail (GLADE_IS_INSPECTOR (inspector));
 	g_return_if_fail (GLADE_IS_PROJECT (project));
 	g_return_if_fail (inspector->priv->project == project);
@@ -822,42 +534,42 @@ project_selection_changed_cb (GladeProject     *project,
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (inspector->priv->view));
 	g_return_if_fail (selection != NULL);
-	
+
 	model = inspector->priv->filter;
 
 	gtk_tree_selection_unselect_all (selection);
 
 	for (list = glade_project_selection_get (project);
 	     list && list->data; list = list->next)
-	{	
+	{
 		if ((widget = glade_widget_get_from_gobject (G_OBJECT (list->data))) != NULL)
 		{
-			if ((iter = glade_util_find_iter_by_widget (model, widget, WIDGET_COLUMN)) != NULL)
+			if ((iter = glade_util_find_iter_by_widget (model, widget, GLADE_PROJECT_MODEL_COLUMN_OBJECT)) != NULL)
 			{
 				path = gtk_tree_model_get_path (model, iter);
 				ancestor_path = gtk_tree_path_copy (path);
-							
-				/* expand parent node */ 
+
+				/* expand parent node */
 				if (gtk_tree_path_up (ancestor_path))
 					gtk_tree_view_expand_to_path
-						(GTK_TREE_VIEW (inspector->priv->view), ancestor_path);			
-			
+						(GTK_TREE_VIEW (inspector->priv->view), ancestor_path);
+
 				gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (inspector->priv->view),
-							      path, 
+							      path,
 							      NULL,
 							      TRUE,
 							      0.5,
 							      0);
-			
-				gtk_tree_selection_select_iter (selection, iter);				
-				
+
+				gtk_tree_selection_select_iter (selection, iter);
+
 				gtk_tree_iter_free (iter);
 				gtk_tree_path_free (path);
 				gtk_tree_path_free (ancestor_path);
 			}
 		}
 	}
-	
+
 	g_signal_handlers_unblock_by_func (gtk_tree_view_get_selection (GTK_TREE_VIEW (inspector->priv->view)),
 					   G_CALLBACK (selection_changed_cb),
 					   inspector);
@@ -869,12 +581,12 @@ selection_foreach_func (GtkTreeModel *model,
 		        GtkTreeIter  *iter, 
 		        gpointer      data)
 {
-	GladeWidget *widget;
+	GObject* object;
 	
-	gtk_tree_model_get (model, iter, WIDGET_COLUMN, &widget, -1);
+	gtk_tree_model_get (model, iter, GLADE_PROJECT_MODEL_COLUMN_OBJECT, &object, -1);
 
-	if (widget)
-		glade_app_selection_add (glade_widget_get_object (widget), FALSE);
+	if (object)
+		glade_app_selection_add (object, FALSE);
 }
 
 static void
@@ -883,20 +595,19 @@ selection_changed_cb (GtkTreeSelection *selection,
 {
 	g_signal_handlers_block_by_func (inspector->priv->project,
 					 G_CALLBACK (project_selection_changed_cb),
-					 inspector); 
-
+					 inspector);
+	
 	glade_app_selection_clear (FALSE);
 	
 	gtk_tree_selection_selected_foreach (selection,
 					     selection_foreach_func,
 					     inspector);
 	glade_app_selection_changed ();
-	
+
 	g_signal_handlers_unblock_by_func (inspector->priv->project,
 					   G_CALLBACK (project_selection_changed_cb),
 					   inspector);
 
-
 	g_signal_emit (inspector, glade_inspector_signals[SELECTION_CHANGED], 0);
 }
 
@@ -915,6 +626,7 @@ button_press_cb (GtkWidget      *widget,
 		 GladeInspector *inspector)
 {
 	GtkTreeView      *view = GTK_TREE_VIEW (widget);
+	GladeInspectorPrivate *priv = inspector->priv;
 	GtkTreePath      *path      = NULL;
 	gboolean          handled   = FALSE;
 
@@ -927,17 +639,17 @@ button_press_cb (GtkWidget      *widget,
 					   NULL, NULL) && path != NULL)
 		{
 			GtkTreeIter  iter;
-			GladeWidget *widget = NULL;
-			if (gtk_tree_model_get_iter (GTK_TREE_MODEL (inspector->priv->model),
+			GladeWidget *object = NULL;
+			if (gtk_tree_model_get_iter (GTK_TREE_MODEL (priv->project),
 							 &iter, path))
 			{
 				/* now we can obtain the widget from the iter.
 				 */
-				gtk_tree_model_get (GTK_TREE_MODEL (inspector->priv->model), &iter,
-						    WIDGET_COLUMN, &widget, -1);
+				gtk_tree_model_get (GTK_TREE_MODEL (priv->project), &iter,
+						    GLADE_PROJECT_MODEL_COLUMN_OBJECT, &object, -1);
 
 				if (widget != NULL)
-					glade_popup_widget_pop (widget, event, TRUE);
+					glade_popup_widget_pop (glade_widget_get_from_gobject (object), event, TRUE);
 				else
 					glade_popup_simple_pop (event);
 				
@@ -955,136 +667,71 @@ button_press_cb (GtkWidget      *widget,
 	return handled;
 }
 
-typedef enum 
-{
-	CELL_ICON,
-	CELL_NAME,
-	CELL_MISC
-} CellType;
-
-static void
-glade_inspector_cell_function (GtkTreeViewColumn *tree_column,
-			       GtkCellRenderer   *cell,
-			       GtkTreeModel      *tree_model,
-			       GtkTreeIter       *iter,
-			       gpointer           data)
-{
-	CellType     type = GPOINTER_TO_INT (data);
-	GladeWidget *widget;
-	gchar       *icon_name, *text = NULL, *child_type, *title;
-
-	gtk_tree_model_get (tree_model, iter, 
-			    TITLE_COLUMN, &title,
-			    WIDGET_COLUMN, &widget, 
-			    -1);
-
-	/* The cell exists, but no widget or title has been associated with it */
-	if (!GLADE_IS_WIDGET (widget) && !title)
-		return;
-
-	switch (type) 
-	{
-	case CELL_ICON:
-		if (widget)
-		{
-			g_object_get (widget->adaptor, "icon-name", &icon_name, NULL);
-			g_object_set (G_OBJECT (cell), "icon-name", icon_name, NULL);
-			g_free (icon_name);
-		}
-		else
-			g_object_set (G_OBJECT (cell), "icon-name", NULL, NULL);
-		break;
-		
-	case CELL_NAME:
-		if (widget)
-			g_object_set (G_OBJECT (cell), 
-				      "text", widget->name, 
-				      "weight", PANGO_WEIGHT_NORMAL,
-				      NULL);
-		else if (title)
-			g_object_set (G_OBJECT (cell), 
-				      "text", title, 
-				      "weight", PANGO_WEIGHT_BOLD,
-				      NULL);
-		else
-			g_object_set (G_OBJECT (cell), 
-				      "text", "dummy", 
-				      "weight", PANGO_WEIGHT_NORMAL,
-				      NULL);
-		break;
-	case CELL_MISC:
-		if (widget)
-		{
-			/* special child type / internal child */
-			if (glade_widget_get_internal (widget) != NULL)
-				text = g_strdup_printf (_("(internal %s)"),  
-							glade_widget_get_internal (widget));
-			else if ((child_type = g_object_get_data (glade_widget_get_object (widget),
-								  "special-child-type")) != NULL)
-				text = g_strdup_printf (_("(%s child)"), child_type);
-			
-			g_object_set (G_OBJECT (cell), "text", text ? text : " ", NULL);
-			if (text) g_free (text);
-		}
-		else
-			g_object_set (G_OBJECT (cell), "text", " ", NULL);
-		break;		
-	default:
-		break;
-	}
-}
-
 static void
 add_columns (GtkTreeView *view)
 {
 	GtkTreeViewColumn *column;
-	GtkCellRenderer *renderer;
+	GtkCellRenderer *renderer_pixbuf, *renderer_name, *renderer_type;
 
 	column = gtk_tree_view_column_new ();
-	
-	/* Class icon */
-	renderer = gtk_cell_renderer_pixbuf_new ();
-	gtk_tree_view_column_pack_start (column, renderer, FALSE);
-	gtk_tree_view_column_set_cell_data_func (column, renderer,
-						 glade_inspector_cell_function,
-						 GINT_TO_POINTER (CELL_ICON), NULL);
 
-	/* Class name */
-	renderer = gtk_cell_renderer_text_new ();
-	g_object_set (G_OBJECT (renderer), 
-		      "xpad", 6, NULL);
-	gtk_tree_view_column_pack_start (column, renderer, FALSE);
-	gtk_tree_view_column_set_cell_data_func (column, renderer,
-						 glade_inspector_cell_function,
-						 GINT_TO_POINTER (CELL_NAME), NULL);	
-	
-	/* Misc internal/special-type */
-	renderer = gtk_cell_renderer_text_new ();
-	g_object_set (G_OBJECT (renderer), 
-		      "style", PANGO_STYLE_ITALIC,
-		      "foreground", "Gray", NULL);
-	gtk_tree_view_column_pack_start (column, renderer, TRUE);
-	gtk_tree_view_column_set_cell_data_func (column, renderer,
-						 glade_inspector_cell_function,
-						 GINT_TO_POINTER (CELL_MISC), NULL);
-	
-	gtk_tree_view_append_column (view, column);	
-		
+	renderer_pixbuf = gtk_cell_renderer_pixbuf_new ();
+	gtk_tree_view_column_pack_start (column, renderer_pixbuf, FALSE);
+	gtk_tree_view_column_set_attributes (column,
+	                                     renderer_pixbuf,
+	                                     "icon_name", GLADE_PROJECT_MODEL_COLUMN_ICON_NAME,
+	                                     NULL);
+
+	renderer_name = gtk_cell_renderer_text_new ();
+	gtk_tree_view_column_pack_start (column, renderer_name, FALSE);
+	gtk_tree_view_column_set_attributes (column,
+	                                     renderer_name,
+	                                     "text", GLADE_PROJECT_MODEL_COLUMN_NAME,
+	                                     NULL);
+
+	renderer_type = gtk_cell_renderer_text_new ();
+	g_object_set (renderer_type, "style", PANGO_STYLE_ITALIC, NULL);
+	gtk_tree_view_column_pack_start (column, renderer_type, FALSE);	
+	gtk_tree_view_column_set_attributes (column,
+	                                     renderer_type,
+	                                     "text", GLADE_PROJECT_MODEL_COLUMN_TYPE_NAME,
+	                                     NULL);
+
+	gtk_tree_view_append_column (view, column);
 	gtk_tree_view_set_headers_visible (view, FALSE);
 }
 
+static void
+update_project_completion (GladeProject    *project,
+                           GladeWidget     *widget,
+                           GladeInspector  *inspector)
+{
+	GladeInspectorPrivate *priv = inspector->priv;
+	const GList* items;
+
+	g_completion_clear_items (priv->completion);
+
+	if (!priv->project)
+		return;
+
+	items = glade_project_get_objects (priv->project);
+
+	/* GCompletion API should take 'const GList *' */
+	g_completion_add_items (priv->completion, (GList *)items);
+}
+
 static void
 disconnect_project_signals (GladeInspector *inspector,
 			    GladeProject   *project)
 {
 	g_signal_handlers_disconnect_by_func (G_OBJECT (project),
-					      G_CALLBACK (project_add_widget_cb),
+					      G_CALLBACK (update_project_completion),
 					      inspector);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (project),
-					      G_CALLBACK (project_remove_widget_cb),
+					      G_CALLBACK (update_project_completion),
 					      inspector);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (project),
-					      G_CALLBACK (project_widget_name_changed_cb),
+					      G_CALLBACK (update_project_completion),
 					      inspector);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (project),
 					      G_CALLBACK (project_selection_changed_cb),
@@ -1096,13 +743,13 @@ connect_project_signals (GladeInspector *inspector,
 			 GladeProject   *project)
 {
 	g_signal_connect (G_OBJECT (project), "add-widget",
-			  G_CALLBACK (project_add_widget_cb),
+			  G_CALLBACK (update_project_completion),
 			  inspector);
 	g_signal_connect (G_OBJECT (project), "remove-widget",
-			  G_CALLBACK (project_remove_widget_cb),
+			  G_CALLBACK (update_project_completion),
 			  inspector);
 	g_signal_connect (G_OBJECT (project), "widget-name-changed",
-			  G_CALLBACK (project_widget_name_changed_cb),
+			  G_CALLBACK (update_project_completion),
 			  inspector);
 	g_signal_connect (G_OBJECT (project), "selection-changed",
 			  G_CALLBACK (project_selection_changed_cb),
@@ -1124,22 +771,32 @@ glade_inspector_set_project (GladeInspector *inspector,
 	g_return_if_fail (GLADE_IS_INSPECTOR (inspector));
 	g_return_if_fail (GLADE_IS_PROJECT (project) || project == NULL);
 
+	GladeInspectorPrivate* priv = inspector->priv;
+
 	if (inspector->priv->project)
 	{
-		disconnect_project_signals (inspector, inspector->priv->project);
-		g_object_unref (inspector->priv->project);
-		inspector->priv->project = NULL;
+		disconnect_project_signals (inspector, project);
+		g_object_unref (priv->project);
+		priv->project = NULL;
 	}
 	
 	if (project)
 	{		
-		inspector->priv->project = project;
-		g_object_ref (inspector->priv->project);
-		connect_project_signals (inspector, inspector->priv->project);		
+		priv->project = project;
+		g_object_ref (priv->project);
+
+		priv->filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (priv->project), NULL);	
+
+		gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (priv->filter),
+						(GtkTreeModelFilterVisibleFunc)glade_inspector_visible_func,
+						inspector, NULL);
+
+		gtk_tree_view_set_model (GTK_TREE_VIEW (priv->view), priv->filter);
+		connect_project_signals (inspector, project);
 	}
 
-	update_model (inspector);
-
+	update_project_completion (project, NULL, inspector);
+	
 	gtk_tree_view_expand_all (GTK_TREE_VIEW (inspector->priv->view));
 	
 	g_object_notify (G_OBJECT (inspector), "project");
@@ -1174,22 +831,25 @@ glade_inspector_get_selected_items (GladeInspector *inspector)
 {
 	GtkTreeSelection* selection;
 	GList *items = NULL, *paths;
+	GladeInspectorPrivate *priv = inspector->priv;
 	
-	
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (inspector->priv->view));
-	
-	paths = gtk_tree_selection_get_selected_rows (selection, NULL);
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->view));
 						      
-	for (; paths; paths = paths->next)
+	for (paths = gtk_tree_selection_get_selected_rows (selection, NULL); 
+	     paths != NULL; paths = g_list_next (paths->next))
 	{
+		GtkTreeIter filter_iter;
 		GtkTreeIter iter;
 		GtkTreePath *path = (GtkTreePath *) paths->data;
-		GladeWidget *widget = NULL;
+		GObject *object = NULL;
 		
-		gtk_tree_model_get_iter (GTK_TREE_MODEL (inspector->priv->model), &iter, path);
-		gtk_tree_model_get (GTK_TREE_MODEL (inspector->priv->model), &iter, WIDGET_COLUMN, &widget, -1);
+		gtk_tree_model_get_iter (priv->filter, &filter_iter, path);
+		gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER(priv->filter),
+		                                                  &iter, &filter_iter);
+		gtk_tree_model_get (GTK_TREE_MODEL (priv->project), &iter, 
+		                    GLADE_PROJECT_MODEL_COLUMN_OBJECT, &object, -1);
 		
-		items = g_list_prepend (items, widget);
+		items = g_list_prepend (items, glade_widget_get_from_gobject (object));
 	}
 	
 	g_list_foreach (paths, (GFunc) gtk_tree_path_free, NULL);
@@ -1231,7 +891,7 @@ glade_inspector_new_with_project (GladeProject *project)
 
 	/* Make sure we expended to the right path */
 	project_selection_changed_cb (project, inspector);
-
+	
 	return GTK_WIDGET (inspector);
 }
 
