@@ -6,6 +6,9 @@
 
 G_BEGIN_DECLS
 
+#define GLADE_DEVHELP_ICON_NAME           "devhelp"
+#define GLADE_DEVHELP_FALLBACK_ICON_FILE  "devhelp.png"
+
 typedef enum _GladeUtilFileDialogType
 {
         GLADE_FILE_DIALOG_ACTION_OPEN,
@@ -34,6 +37,10 @@ void		glade_util_flash_message	(GtkWidget *statusbar,
 						 guint context_id,
 						 gchar *format, ...);
 
+gboolean        glade_util_check_and_warn_scrollable (GladeWidget        *parent,
+						      GladeWidgetAdaptor *child_adaptor,
+						      GtkWidget          *parent_widget);
+
 /* This is a GCompareFunc for comparing the labels of 2 stock items, ignoring
    any '_' characters. It isn't particularly efficient. */
 
@@ -152,6 +159,8 @@ gint              glade_utils_hijack_key_press (GtkWindow          *win,
 						gpointer            user_data);
 	
 
+gchar           *glade_utils_replace_home_dir_with_tilde (const gchar *uri);
+
 G_END_DECLS
 
 #endif /* __GLADE_UTILS_H__ */
